#!/usr/bin/env bash

script_name="$(basename $(realpath $0))"
script_dir="$(dirname $(realpath $0))"

conf_dir="${script_dir}/conf"

declare -a unsorted_website_arr
declare -a website_arr

log_file="$(mktemp)"

backup_date=$(date +%Y-%m-%d-%H%M)


# =============
# --- Some functions
# =============

clean_up() {


   if [[ -f "$_backup_crontab_file" ]]; then

      echononl "(Re)Install previously saved crontab from '$_backup_crontab_file'.."

      crontab $_backup_crontab_file >> $log_file 2>&1

      if [[ $? -eq 0 ]]; then
         echo_ok
      else
         echo_failed
			error "$(cat $log_file)"
      fi

   fi

   # Perform program exit housekeeping
   rm -f $log_file
   blank_line
   exit $1
}

is_number() {

   return $(test ! -z "${1##*[!0-9]*}" > /dev/null 2>&1);

   # - also possible
   # -
   #[[ ! -z "${1##*[!0-9]*}" ]] && return 0 || return 1
   #return $([[ ! -z "${1##*[!0-9]*}" ]])
}

echononl(){
   echo X\\c > /tmp/shprompt$$
   if [ `wc -c /tmp/shprompt$$ | awk '{print $1}'` -eq 1 ]; then
      echo -e -n "$*\\c" 1>&2
   else
       echo -e -n "$*" 1>&2
   fi
   rm /tmp/shprompt$$
}
echo_done() {
   if $terminal ; then
      echo -e "\033[75G[ \033[32mdone\033[m ]"
   else
      echo " [ done ]"
   fi
}
echo_ok() {
   if $terminal ; then
      echo -e "\033[75G[ \033[32mok\033[m ]"
   else
      echo " [ ok ]"
   fi
}
echo_warning() {
   if $terminal ; then
      echo -e "\033[75G[ \033[33m\033[1mwarn\033[m ]"
   else
      echo " [ warning ]"
   fi
}
echo_failed(){
   if $terminal ; then
      echo -e "\033[75G[ \033[1;31mfailed\033[m ]"
   else
      echo ' [ failed! ]'
   fi
}
echo_skipped() {
   if $terminal ; then
      echo -e "\033[75G[ \033[37mskipped\033[m ]"
   else
      echo " [ skipped ]"
   fi
}
fatal (){
   echo ""
   echo ""
   if $terminal ; then
      echo -e "\t[ \033[31m\033[1mFatal\033[m ]: \033[37m\033[1m$*\033[m"
      echo ""
      echo -e "\t\033[31m\033[1m           Script will be interrupted..\033[m\033[m"
   else
      echo "fatal: $*"
      echo "Script will be interrupted.."
   fi
   clean_up 1
}

error(){
   echo ""
   if $terminal ; then
      echo -e "\t[ \033[31m\033[1mFehler\033[m ]: $*"
   else
      echo "Error: $*"
   fi
   echo ""
}

warn (){
   echo ""
   if $terminal ; then
      echo -e "\t[ \033[33m\033[1mWarning\033[m ]: $*"
   else
      echo "Warning: $*"
   fi
   echo ""
}

info (){
   echo ""
   if $terminal ; then
      echo -e "\t[ \033[32m\033[1mInfo\033[m ]: $*"
   else
      echo "Info: $*"
   fi
   echo ""
}

## - Check if a given array (parameter 2) contains a given string (parameter 1)
## -
containsElement () {
   local e
   for e in "${@:2}"; do [[ "$e" == "$1" ]] && return 0; done
   return 1
}

blank_line() {
   if $terminal ; then
      echo ""
   fi
}



# ----------
# - Jobhandling 
# ----------

# - Run 'clean_up' for signals SIGHUP SIGINT SIGTERM
# -
trap clean_up SIGHUP SIGINT SIGTERM

## - 
while IFS='' read -r -d '' _conf_file ; do
   source $_conf_file
   if [[ -n "$WEBSITE" ]] ; then
      unsorted_website_arr+=("${WEBSITE}:$_conf_file")
   fi
   WEBSITE=""
done  < <(find "${conf_dir}" -maxdepth 1 -type f -name "*.conf" -print0)

if [[ ${#unsorted_website_arr} -eq 0 ]]; then
   fatal "No configuration files found in '${script_dir}/conf' or no website configured!"
fi

# - Sort array
# -
IFS=$'\n' website_arr=($(sort <<<"${unsorted_website_arr[*]}"))



# =============
# --- Some 
# =============

# - Support systemd ?
# -
if [[ "X$(which systemd)" = "X" ]]; then
   SYSTEMD_EXISTS=false
else
   SYSTEMD_EXISTS=true
fi

# - Running in a terminal?
# -
if [[ -t 1 ]] ; then
      terminal=true
else
   terminal=false
fi

#clear

echo ""
echo -e "\033[32m-----\033[m"
echo "Update Nextcloud"
echo -e "\033[32m-----\033[m"


WEBSITE=
_OK=false
echo ""
echo "Which site would you like to restore?"
echo ""
declare -i i=0
for _site in ${website_arr[@]} ; do
   IFS=':' read -a _arr <<< ${_site}
   echo "   [$i] ${_arr[0]}"
   ((i++))
done
echo
echononl "   Eingabe: "
while ! $_OK ; do
read _IN
   if is_number "$_IN" && [[ -n ${website_arr[$_IN]} ]]; then
      IFS=':' read -a _arr <<< ${website_arr[$_IN]}
      conf_file=${_arr[1]}
      _OK=true
   else
      echo ""  
      echo -e "\tFalsche Eingabe !"
      echo ""
      echononl "   Eingabe: "
   fi
done


echo ""
echononl "   Include Configuration file.."
if [[ ! -f $conf_file ]]; then
   echo_failed
   fatal "Missing configuration file '$conf_file'"
else
   source $conf_file
   echo_ok
fi
echo ""



# =============
# --- Some checks
# =============

DEFAULT_WEB_BASE_DIR="/var/www/$WEBSITE"
DEFAULT_HTTP_USER="www-data"
DEFAULT_HTTP_GROUP="www-data"


if [[ -z ${WEBSITE} ]] ; then
   fatal "No website given (parameter 'WEBSITE')"
fi

[[ -n "$WEB_BASE_DIR" ]] || WEB_BASE_DIR=$DEFAULT_WEB_BASE_DIR

if [[ ! -d ${WEB_BASE_DIR} ]] ; then
   fatal "Web base directory not found (parameter 'WEB_BASE_DIR')"
fi

if [[ "$DATABASE_TYPE" != "postgres" ]] && [[ "$DATABASE_TYPE" != "mysql" ]]; then
   fatal "Wrong or missing database type (parameter 'DATABASE_TYPE')"
fi

if [[ -z "$DATABASE_NAME" ]]; then
   fatal "Missing database name (parameter 'DATABASE_NAME')"
fi

if [[ "$DATABASE_TYPE" = "mysql" ]] && [[ -z "$MYSQL_CREDENTIALS" ]]; then
   fatal "No Database Credentials for MySQL given (parameter 'MYSQL_CREDENTIALS')"
fi

if [[ "$DATABASE_TYPE" = "postgres" ]]; then
   if [[ -z "$PSQL_USER" ]] || [[ -z "$PSQL_PASS" ]]; then
      fatal "No Database Credentials for PostgreSQL given (parameters: 'PSQL_USER' 'PSQL_PASS'"
   fi
fi

# - Determin user/group of the webserver
# -
httpd_binary="`which httpd`"
if [ -z "$httpd_binary" ]; then
   httpd_binary="$(ps -axu | grep httpd |  grep -e "^root" | grep -v grep | awk '{print$11}')"
   if [ -z "$httpd_binary" ]; then
      if [ -x "/usr/local/apache2/bin/httpd" ]; then
         httpd_binary="/usr/local/apache2/bin/httpd"
      fi
   fi
fi
if [ -x "$httpd_binary" ];then

   # - Determin websever user
   # -
   _HTTP_USER="`$httpd_binary -t -D DUMP_RUN_CFG | grep -i -e "^User" | awk '{print$2}' | cut -d\"=\" -f2 | tr -d '"'`"
   _HTTP_GROUP="`$httpd_binary -t -D DUMP_RUN_CFG | grep -i -e "^Group" | awk '{print$2}' | cut -d\"=\" -f2 | tr -d '"'`"

   # - Is webserver running ?
   # -
   PID=$(ps aux | grep "$(realpath $httpd_binary)" | grep -e "^root" | grep -v grep | awk '{print$2}')
   if [[ "X${PID}X" = "XX" ]] ;then
      IS_HTTPD_RUNNING=false
   else
      IS_HTTPD_RUNNING=true
   fi
fi

if [[ -n "$_HTTP_USER" ]] ; then
   if [[ -n "$HTTP_USER" ]] && [[ "$_HTTP_USER" != "$HTTP_USER" ]]; then
      warn "The script has determined \033[1;37m${_HTTP_USER}\033[m as Webservers user. This\n                     value differs from the value given in your configuration file, \n                     which is \033[1;37m${HTTP_USER}\033[m and takes precedence."
   else
      HTTP_USER=$_HTTP_USER
   fi
else
   [[ -n "$HTTP_USER" ]] || HTTP_USER=$DEFAULT_HTTP_USER
fi
if [[ -n "$_HTTP_GROUP" ]] ; then
   if [[ -n "$HTTP_GROUP" ]] && [[ "$_HTTP_GROUP" != "$HTTP_GROUP" ]]; then
      warn "The script has determined \033[1;37m${_HTTP_GROUP}\033[m as Webservers group. This\n                     value differs from the value given in your configuration file, \n                     which is \033[1;37m${HTTP_GROUP}\033[m and takes precedence."
   else
      HTTP_GROUP=$_HTTP_GROUP
   fi
else
   [[ -n "$HTTP_GROUP" ]] || HTTP_GROUP=$DEFAULT_HTTP_GROUP
fi


# - Get available backup versions 
# -
for _dir in $(ls -d ${WEB_BASE_DIR}/nextcloud-*) ; do
   if [[ "$(basename "$_dir")" =~ ^nextcloud-[0-9]{1,2}\.[0-9]{1,2}\.[0-9]{1,2}\.[0-9]{4} ]]; then
      _version="$(echo "$_dir"| grep -o -E "[0-9]{1,2}\.[0-9]{1,2}\.[0-9]{1,2}")"
      _backup_date="$(echo "$_dir"| grep -o -E "[0-9]{4}-[0-9]{2}-[0-9]{2}-[0-9]{4}")"
      if [[ -d "${WEB_BASE_DIR}/nextcloud-${_version}.${_backup_date}" ]] \
         && [[ -d "${WEB_BASE_DIR}/data-${_version}.${_backup_date}" ]] \
         && [[ -f "${WEB_BASE_DIR}/${DATABASE_NAME}-v${_version}.${_backup_date}.sql" ]]; then

         nextcloud_version_arr+=("$(basename "${_dir}"):${_version}:${_backup_date}")
      fi
   fi
done

if [[ ${#nextcloud_version_arr[@]} -lt 1 ]] ; then
   fatal "No backuped version present!"
fi


_OK=false
echo ""
echo "Which version would you like to restore?"
echo ""
declare -i i=0
for _backup_version in ${nextcloud_version_arr[@]} ; do
   IFS=':' read -a _arr <<< ${_backup_version}
   echo "   [$i] Version ${_arr[1]} from ${_arr[2]}"
   ((i++))
done
echo
echononl "   Eingabe: "
while ! $_OK ; do
read _IN
   if is_number "$_IN" && [[ -n ${nextcloud_version_arr[$_IN]} ]]; then
      IFS=':' read -a _arr <<< ${nextcloud_version_arr[$_IN]}
      RESTORE_VERSION="${_arr[1]}"
      DATE_RESTORE_VERSION="${_arr[2]}"
      _OK=true
   else
      echo ""  
      echo -e "\tFalsche Eingabe !"
      echo ""
      echononl "   Eingabe: "
   fi
done

CURRENT_INSTALL_DIR=`realpath ${WEB_BASE_DIR}/nextcloud`
CURRENT_DATA_DIR=`realpath ${WEB_BASE_DIR}/data`
CURRENT_VERSION=`basename $CURRENT_INSTALL_DIR | cut -d"-" -f2`


echo ""
echo -e "\033[1;32mStarting Nextcloud Restore for \033[1;37m${WEBSITE}\033[m"
echo ""
echo -e   "   Website to restore...................: $WEBSITE"
echo ""
echo -e   "   Current version of nextcloud.........: $CURRENT_VERSION"
echo ""
echo -e   "   Version to rstore....................: $RESTORE_VERSION"
echo -e   "   Backup date was......................: $DATE_RESTORE_VERSION"
echo ""
echo -e   "   Databse name.........................: $DATABASE_NAME"
echo -e   "   Database type........................: $DATABASE_TYPE"
echo ""
if [[ "$DATABASE_TYPE" = "mysql" ]] ; then
   echo -e   "   Mysql Credentials....................: $MYSQL_CREDENTIALS"
fi
echo ""

echo ""
echo -n "   Type upper case 'YES' to continue executing with this parameters: "
read OK
if [[ "$OK" = "YES" ]] ; then
   echo ""
   echo ""
   echo -e "\033[1;32mGoing to restore Nextcloud to Version '$RESTORE_VERSION' on \033[1;37m$WEBSITE \033[m"
   echo ""
else
   fatal "Abort by user request - Answer as not 'YES'"
fi


echo ""

DATA_DIR=${WEB_BASE_DIR}/data-${VERSION}
INSTALL_DIR=${WEB_BASE_DIR}/nextcloud-${VERSION}

# - Deaktiviere Cronjobs
# -
_backup_crontab_file=/tmp/crontab_root.${backup_date}
echononl "   Backup Crontab to '$_backup_crontab_file'"
crontab -l > $_backup_crontab_file 2> $log_file
if [[ $? -eq 0 ]]; then
   echo_ok
else
   echo_failed
   fatal "$(cat $log_file)"
fi

echononl "   Remove crontab for root.."
crontab -r > $log_file 2>&1
if [[ $? -eq 0 ]]; then
   echo_ok
else
   echo_failed
   fatal "$(cat $log_file)"
fi


# -----
# - Doing some pre-script tasks
# -----

echo ""
echo ""
echo -e "\033[37m\033[1mDoing some pre-script tasks..\033[m"


# - Stop Apache Webserver
# -
echo ""
echononl "   Stop Apache Webserver.."
if $SYSTEMD_EXISTS ; then
   systemctl stop apache2
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      fatal "$(cat $log_file)"
   fi
else
   /etc/init.d/apache2 stop
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      fatal "$(cat $log_file)"
   fi
fi


# -----
# - Backup/Save current Installation
# -----

echo ""
echo ""
echo -e "\033[37m\033[1mBackup/Save current Installation..\033[m"
echo ""

# - Backup Database
# -
echononl "   Backup MySQL database '$DATABASE_NAME'.."
if [[ "$DATABASE_TYPE" = 'mysql' ]]; then
   mysqldump $MYSQL_CREDENTIALS --opt $DATABASE_NAME > \
      ${WEB_BASE_DIR}/${DATABASE_NAME}-v${CURRENT_VERSION}.${backup_date}.sql 2> $log_file
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      fatal "$(cat $log_file)"
   fi
elif [[ "$DATABASE_TYPE" = 'postgres' ]]; then
   PGPASSWORD=$PSQL_PASS \
   pg_dump $DATABASE_NAME -h $PSQL_SERVER -U $PSQL_USER \
      -f ${WEB_BASE_DIR}/${DATABASE_NAME}-v${CURRENT_VERSION}.${backup_date}.sql 2> $log_file
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      fatal "$(cat $log_file)"
   fi
fi


if [[ "$DATABASE_TYPE" = 'mysql' ]]; then

	echononl "   Get table list from database '$DATABASE_NAME' .."
	_TABLES="$(mysql $MYSQL_CREDENTIALS $DATABASE_NAME -N -s -e "show tables" 2> $log_file)"
	if [[ $? -ne 0 ]] ; then
		echo_failed
		fatal "$(cat $log_file)"
	else
		echo_ok
	fi

	if [[ -z "$_TABLES" ]]; then

		warn "No tables found in database '$DATABASE_NAME'."

	else

      echononl "   Drop tables from database '$DATABASE_NAME' .."
      _failed=false
      > $log_file
      for _table in $_TABLES ; do
         mysql $MYSQL_CREDENTIALS $DATABASE_NAME -N -s -e "DROP TABLE \`$_table\`" >> $log_file 2>&1
         if [[ $? -ne 0 ]] ; then
            _failed=true
            _tables_not_deleted+=("$_table")
         fi
      done
      if $_failed ; then

         echo_failed
         error "$(cat $log_file)"

         if [[ ${#_tables_not_deleted[@]} -gt 0 ]] ; then
            echo ""
            echo "Tables NOT deleted:"
            for _table in "${_tables_not_deleted[@]}" ; do
               echo  "   $_table"
            done
            echo ""
         fi
      else
         echo_ok
      fi

   fi

fi


# - Backup current data directory
# -
echononl "   Backup current data directory"
_current_data_dir="$(realpath ${WEB_BASE_DIR}/data)"
mv "$_current_data_dir" "${_current_data_dir}.$backup_date"  > $log_file 2>&1
if [[ $? -eq 0 ]]; then
   echo_ok
else
   echo_failed
   error "$(cat $log_file)"
fi


# - Backup old installation directory
# -
echononl "   Backup current installation directory"
_current_install_dir="$(realpath ${WEB_BASE_DIR}/nextcloud)"
mv "$_current_install_dir" "${_current_install_dir}.$backup_date" > $log_file 2>&1
if [[ $? -eq 0 ]]; then
   echo_ok
else
   echo_failed
   error "$(cat $log_file)"
fi


echo ""


# -----
# - Backup/Save current Installation
# -----

echo ""
echo ""
echo -e "\033[37m\033[1mRestore to Version '$RESTORE_VERSION'..\033[m"
echo ""


# - Delete symlink for current data directory
# -
echononl "   Remove symlink from current data dir 'data-${CURRENT_VERSION}'.."
rm ${WEB_BASE_DIR}/data  > $log_file 2>&1
if [[ $? -eq 0 ]]; then
   echo_ok
else
   echo_failed
   fatal "$(cat $log_file)"
fi


# - Remove symlink from current installation directory
# -
echononl "   Remove symlink from current installation directory"
rm ${WEB_BASE_DIR}/nextcloud > $log_file 2>&1
if [[ $? -eq 0 ]]; then
   echo_ok
else
   echo_failed
   fatal "$(cat $log_file)"
fi


echo ""


# - Move installation dir of backuped version to current installation dir
# -
echononl "   Move backuped install dir to current install dir"
mv "${WEB_BASE_DIR}/nextcloud-${RESTORE_VERSION}.${DATE_RESTORE_VERSION}" \
   "${WEB_BASE_DIR}/nextcloud-${RESTORE_VERSION}" > $log_file 2>&1
if [[ $? -eq 0 ]]; then
   echo_ok
else
   echo_failed
   fatal "$(cat $log_file)"
fi


# - Move data dir of backuped version to current data dir
# -
echononl "   Move backuped data dir to current data dir"
mv "${WEB_BASE_DIR}/data-${RESTORE_VERSION}.${DATE_RESTORE_VERSION}" \
   "${WEB_BASE_DIR}/data-${RESTORE_VERSION}" > $log_file 2>&1
if [[ $? -eq 0 ]]; then
   echo_ok
else
   echo_failed
   fatal "$(cat $log_file)"
fi


# - Restore Database
# -
echononl "   Restore Database"
if [[ "$DATABASE_TYPE" = 'mysql' ]]; then
   mysql $MYSQL_CREDENTIALS $DATABASE_NAME < \
      ${WEB_BASE_DIR}/${DATABASE_NAME}-v${RESTORE_VERSION}.${DATE_RESTORE_VERSION}.sql 2> $log_file
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      fatal "$(cat $log_file)"
   fi
elif [[ "$DATABASE_TYPE" = 'postgres' ]]; then
   PGPASSWORD=$PSQL_PASS \
   pg_dump $DATABASE_NAME -h $PSQL_SERVER -U $PSQL_USER \
      -f ${WEB_BASE_DIR}/${DATABASE_NAME}-v${RESTORE_VERSION}.${DATE_RESTORE_VERSION}.sql 2> $log_file
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      fatal "$(cat $log_file)"
   fi
fi


echo ""


# - Set symlink for data directory
# -
echononl "   Set symlink for data dir 'data-${RESTORE_VERSION}'.."
ln -s data-${RESTORE_VERSION} ${WEB_BASE_DIR}/data > $log_file 2>&1
if [[ $? -eq 0 ]]; then
   echo_ok
else
   echo_failed
   fatal "$(cat $log_file)"
fi

# - Set symlink (documentroot) for installation directory
# -
echononl "   Set symlink (documentroot) for installation directory"
ln -s nextcloud-${RESTORE_VERSION} ${WEB_BASE_DIR}/nextcloud > $log_file 2>&1
if [[ $? -eq 0 ]]; then
   echo_ok
else
   echo_failed
   fatal "$(cat $log_file)"
fi


# -----
# - Doing some post-script tasks
# -----

echo ""
echo ""
echo -e "\033[37m\033[1mDoing some post-script tasks..\033[m"

echo ""
echononl "   Restart PHP engine.."
if [[ "$PHP_ENGINE" = "FPM" ]]; then
   if $SYSTEMD_EXISTS ; then
      systemctl restart php-${PHP_VERSION}-fpm > $log_file 2>&1
      if [[ $? -eq 0 ]]; then
         echo_ok
      else
         echo_failed
         fatal "$(cat $log_file)"
      fi
   else
      /etc/init.d/php-${PHP_VERSION}-fpm restart > $log_file 2>&1
      if [[ $? -eq 0 ]]; then
         echo_ok
      else
         echo_failed
         fatal "$(cat $log_file)"
      fi
   fi
else
   echo_skipped
fi

# - Start Apache Webserver
# -
echo ""
echononl "   Start Apache Webserver.."
if $IS_HTTPD_RUNNING ; then
   if $SYSTEMD_EXISTS ; then
      systemctl start apache2 > $log_file 2>&1
      if [[ $? -eq 0 ]]; then
         echo_ok
      else
         echo_failed
         fatal "$(cat $log_file)"
      fi
   else
      /etc/init.d/apache2 start > $log_file 2>&1
      if [[ $? -eq 0 ]]; then
         echo_ok
      else
         echo_failed
         fatal "$(cat $log_file)"
      fi
   fi
else
   echo_skipped
   warn "The webserver was not running, so it will be keept down!"
fi

_redis_cli_bin="$(which redis-cli)"
if [[ -z "$_redis_cli_bin" ]]; then
   if [[ -x "/usr/local/bin/redis-cli" ]]; then
      _redis_cli_bin="/usr/local/bin/redis-cli"
   fi
fi


echononl "   Flush redis cache.."
if [[ -x "$_redis_cli_bin" ]]; then
   $_redis_cli_bin flushall > $log_file 2>&1
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      error "$(cat $log_file)"
   fi
else
   echo_skipped
fi


echononl "   Restart redis server.."
if $SYSTEMD_EXISTS ; then
   systemctl restart redis-server > $log_file 2>&1
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      error "$(cat $log_file)"
   fi
else
   /etc/init.d/redis-server restart  > $log_file 2>&1
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      error "$(cat $log_file)"
   fi
fi

blank_line
clean_up 0


cd $WEB_BASE_DIR


------------------------------------------------------------------------

## - Alternativly enable maintenance mode
## -
su -c "/usr/local/php/bin/php ${WEB_BASE_DIR}/htdocs/occ maintenance:mode --on" -s /bin/bash $HTTP_USER



## - Deactivate third party apps
## -
## - for _app in $THIRD_PARTY_APPS ; do
## -    #su -c"/usr/local/php/bin/php ${WEB_BASE_DIR}/htdocs/occ app:disable $_app" -s /bin/bash $HTTP_USER
## -    su -c"/usr/local/php/bin/php ${WEB_BASE_DIR}/nextcloud/occ app:disable $_app" -s /bin/bash $HTTP_USER
## - done
for _app in $THIRD_PARTY_APPS ; do
   #su -c"/usr/local/php/bin/php ${WEB_BASE_DIR}/htdocs/occ app:disable $_app" -s /bin/bash $HTTP_USER
   su -c"/usr/local/php/bin/php ${WEB_BASE_DIR}/nextcloud/occ app:disable $_app" -s /bin/bash $HTTP_USER
done




## - ## - Enable third party apps
## - ## -
## - for _app in $THIRD_PARTY_APPS ; do
## -    su -c"/usr/local/php/bin/php ${WEB_BASE_DIR}/htdocs/occ app:enable $_app" -s /bin/bash $HTTP_USER
## - done
for _app in $THIRD_PARTY_APPS ; do
   su -c"/usr/local/php/bin/php ${WEB_BASE_DIR}/htdocs/occ app:enable $_app" -s /bin/bash $HTTP_USER
done


------------------------------------------------------------------------

https://doc.owncloud.org/server/8.0/admin_manual/maintenance/upgrade.html

while IFS= read -r -d '' n; do
   #if [[ "$n" == "mb" ]]; then
   #   continue
   #fi
   su -c "/usr/local/php/bin/php ${WEB_BASE_DIR}/htdocs/console.php files:scan --path `basename $n`" -s /bin/bash $HTTP_USER
done < <(find $WEB_BASE_DIR/data/ -mindepth 1 -maxdepth 1 -type d -print0)

su -c "/usr/local/php/bin/php ${WEB_BASE_DIR}/htdocs/console.php files:scan --all" -s /bin/bash $HTTP_USER
su -c "/usr/local/php/bin/php ${WEB_BASE_DIR}/htdocs/occ maintenance:repair" -s /bin/bash $HTTP_USER
